@@ -1,6 +1,5 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 
 import ConfigParser
 import gtk
@@ -16,13 +15,14 @@ import tryton.common as common
 from tryton.config import CONFIG, TRYTON_ICON, PIXMAPS_DIR, get_config_dir
 import tryton.rpc as rpc
 from tryton.gui.window.dbcreate import DBCreate
+from tryton.exceptions import TrytonError
 
 _ = gettext.gettext
 
 
 class DBListEditor(object):
 
-    def __init__(self, parent, profile_store, profiles):
+    def __init__(self, parent, profile_store, profiles, callback):
         self.profiles = profiles
         self.current_database = None
         self.old_profile, self.current_profile = None, None
@@ -85,6 +85,7 @@ class DBListEditor(object):
         self.port_entry = gtk.Entry()
         self.port_entry.connect('focus-out-event', self.display_dbwidget)
         self.port_entry.connect('changed', self.update_profiles, 'port')
+        self.port_entry.connect('insert_text', self.insert_text_port)
         self.port_entry.set_activates_default(True)
         table.attach(port, 0, 1, 2, 3, yoptions=False, xoptions=gtk.FILL)
         table.attach(self.port_entry, 1, 2, 2, 3, yoptions=False)
@@ -141,6 +142,16 @@ class DBListEditor(object):
         self.dialog.set_default_size(640, 350)
         self.dialog.set_default_response(gtk.RESPONSE_ACCEPT)
 
+        self.dialog.connect('close', lambda *a: False)
+        self.dialog.connect('response', self.response)
+        self.callback = callback
+
+    def response(self, widget, response):
+        if self.callback:
+            self.callback(self.current_profile['name'])
+        self.parent.present()
+        self.dialog.destroy()
+
     def run(self, profile_name):
         self.clear_entries()  # must be done before show_all for windows
         self.dialog.show_all()
@@ -153,10 +164,6 @@ class DBListEditor(object):
                 i = 0
             self.profile_tree.get_selection().select_path((i,))
             self.profile_selected(self.profile_tree)
-        self.dialog.run()
-        self.parent.present()
-        self.dialog.destroy()
-        return self.current_profile['name']
 
     def _current_profile(self):
         model, selection = self.profile_tree.get_selection().get_selected()
@@ -168,7 +175,7 @@ class DBListEditor(object):
         for entryname in ('host', 'port', 'database', 'username'):
             entry = getattr(self, '%s_entry' % entryname)
             if entryname == 'port':
-                entry.set_text('8070')
+                entry.set_text('8000')
             else:
                 entry.set_text('')
         self.current_database = None
@@ -283,6 +290,8 @@ class DBListEditor(object):
         port = self.port_entry.get_text()
         if not (host and port):
             return
+        if self.updating_db:
+            return
         if dbname is None:
             dbname = self.current_database
 
@@ -292,33 +301,40 @@ class DBListEditor(object):
         self.remove_button.set_sensitive(False)
         self.ok_button.set_sensitive(False)
         self.cell.set_property('editable', False)
+        self.host_entry.set_sensitive(False)
+        self.port_entry.set_sensitive(False)
         self.updating_db = True
-        dbs, createdb = dbprogress.update(self.database_combo,
-            self.database_progressbar, dbname)
-        self.updating_db = False
 
-        if dbs is None and createdb is None:
-            pass
-        elif dbs is None or dbs == -1:
-            if dbs is None:
-                label = _(u'Could not connect to the server')
-            else:
-                label = _(u'Incompatible version of the server')
-            self.database_label.set_label('<b>%s</b>' % label)
-            self.database_label.show()
-        elif dbs == 0:
-            if createdb:
-                self.database_button.show()
-            else:
-                self.database_entry.show()
-        else:
-            self.database_entry.set_text(dbname if dbname else '')
-            self.database_combo.show()
+        def callback(dbs, createdb):
+            self.updating_db = False
 
-        self.add_button.set_sensitive(True)
-        self.remove_button.set_sensitive(True)
-        self.ok_button.set_sensitive(True)
-        self.cell.set_property('editable', True)
+            if dbs is None and createdb is None:
+                pass
+            elif dbs is None or dbs == -1:
+                if dbs is None:
+                    label = _(u'Could not connect to the server')
+                else:
+                    label = _(u'Incompatible version of the server')
+                self.database_label.set_label('<b>%s</b>' % label)
+                self.database_label.show()
+            elif dbs == 0:
+                if createdb:
+                    self.database_button.show()
+                else:
+                    self.database_entry.show()
+            else:
+                self.database_entry.set_text(dbname if dbname else '')
+                self.database_combo.show()
+
+            self.add_button.set_sensitive(True)
+            self.remove_button.set_sensitive(True)
+            self.ok_button.set_sensitive(True)
+            self.cell.set_property('editable', True)
+            self.host_entry.set_sensitive(True)
+            self.port_entry.set_sensitive(True)
+
+        dbprogress.update(self.database_combo,
+            self.database_progressbar, callback, dbname)
 
     def db_create(self, button):
         if not self.current_profile['name']:
@@ -326,7 +342,7 @@ class DBListEditor(object):
         host = self.host_entry.get_text()
         port = int(self.port_entry.get_text())
         dia = DBCreate(host, port)
-        dbname = dia.run(self.dialog)
+        dbname = dia.run()
         self.username_entry.set_text('admin')
         self.display_dbwidget(None, None, dbname)
 
@@ -344,11 +360,21 @@ class DBListEditor(object):
             self.profiles.set(self.current_profile['name'], 'database', dbname)
             self.validate_profile(self.current_profile['name'])
 
+    def insert_text_port(self, entry, new_text, new_text_length, position):
+        value = entry.get_text()
+        position = entry.get_position()
+        new_value = value[:position] + new_text + value[position:]
+        try:
+            int(new_value)
+        except ValueError:
+            entry.stop_emission('insert-text')
+
 
 class DBLogin(object):
-    def __init__(self, parent):
+    def __init__(self):
         # GTK Stuffs
-        self.dialog = gtk.Dialog(title=_('Login'), parent=parent,
+        self.parent = common.get_toplevel_window()
+        self.dialog = gtk.Dialog(title=_('Login'), parent=self.parent,
             flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT)
         self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.dialog.set_has_separator(True)
@@ -357,7 +383,7 @@ class DBLogin(object):
         tooltips = common.Tooltips()
         button_cancel = gtk.Button(_('_Cancel'))
         img_cancel = gtk.Image()
-        img_cancel.set_from_stock('tryton-cancel', gtk.ICON_SIZE_BUTTON)
+        img_cancel.set_from_stock('gtk-cancel', gtk.ICON_SIZE_BUTTON)
         button_cancel.set_image(img_cancel)
         tooltips.set_tip(button_cancel,
             _('Cancel connection to the Tryton server'))
@@ -396,8 +422,6 @@ class DBLogin(object):
         self.combo_profile.add_attribute(cell, 'sensitive', 1)
         self.combo_profile.set_model(self.profile_store)
         self.combo_profile.connect('changed', self.profile_changed)
-        self.combo_profile.connect('move-active', self.profile_move_active)
-        self.move_active = False
         self.profile_label = gtk.Label(_(u'Profile:'))
         self.profile_label.set_justify(gtk.JUSTIFY_RIGHT)
         self.profile_label.set_alignment(1, 0.5)
@@ -421,6 +445,7 @@ class DBLogin(object):
         self.label_host.set_padding(3, 3)
         self.entry_host = gtk.Entry()
         self.entry_host.connect_after('focus-out-event', self.clear_profile_combo)
+        self.entry_host.set_activates_default(True)
         self.table_main.attach(self.label_host, 0, 1, 4, 5, xoptions=gtk.FILL)
         self.table_main.attach(self.entry_host, 1, 3, 4, 5)
         self.label_database = gtk.Label(_('Database:'))
@@ -429,6 +454,7 @@ class DBLogin(object):
         self.label_database.set_padding(3, 3)
         self.entry_database = gtk.Entry()
         self.entry_database.connect_after('focus-out-event', self.clear_profile_combo)
+        self.entry_database.set_activates_default(True)
         self.table_main.attach(self.label_database, 0, 1, 5, 6,
             xoptions=gtk.FILL)
         self.table_main.attach(self.entry_database, 1, 3, 5, 6)
@@ -451,13 +477,13 @@ class DBLogin(object):
 
         # Profile informations
         self.profile_cfg = os.path.join(get_config_dir(), 'profiles.cfg')
-        self.profiles = ConfigParser.SafeConfigParser({'port': '8070'})
+        self.profiles = ConfigParser.SafeConfigParser({'port': '8000'})
         if not os.path.exists(self.profile_cfg):
             short_version = '.'.join(VERSION.split('.', 2)[:2])
             name = 'demo%s.tryton.org' % short_version
             self.profiles.add_section(name)
             self.profiles.set(name, 'host', name)
-            self.profiles.set(name, 'port', '8070')
+            self.profiles.set(name, 'port', '8000')
             self.profiles.set(name, 'database', 'demo%s' % short_version)
             self.profiles.set(name, 'username', 'demo')
         else:
@@ -468,20 +494,23 @@ class DBLogin(object):
             self.profile_store.append([section, active])
 
     def profile_manage(self, widget):
-        dia = DBListEditor(self.dialog, self.profile_store, self.profiles)
+        def callback(profile_name):
+            with open(self.profile_cfg, 'wb') as configfile:
+                self.profiles.write(configfile)
+
+            for idx, row in enumerate(self.profile_store):
+                if row[0] == profile_name and row[1]:
+                    self.combo_profile.set_active(idx)
+                    self.profile_changed(self.combo_profile)
+                    break
+
+        dia = DBListEditor(self.dialog, self.profile_store, self.profiles,
+            callback)
         active_profile = self.combo_profile.get_active()
         profile_name = None
         if active_profile != -1:
             profile_name = self.profile_store[active_profile][0]
-        profile_name = dia.run(profile_name)
-        with open(self.profile_cfg, 'wb') as configfile:
-            self.profiles.write(configfile)
-
-        for idx, row in enumerate(self.profile_store):
-            if row[0] == profile_name:
-                self.combo_profile.set_active(idx)
-                self.profile_changed(self.combo_profile)
-                break
+        dia.run(profile_name)
 
     def profile_changed(self, combobox):
         position = combobox.get_active()
@@ -493,27 +522,28 @@ class DBLogin(object):
         except ConfigParser.NoOptionError:
             username = ''
         host = self.profiles.get(profile, 'host')
+        if ':' in host:
+            host = '[%s]' % host
         port = self.profiles.get(profile, 'port')
         self.entry_host.set_text('%s:%s' % (host, port))
         self.entry_database.set_text(self.profiles.get(profile, 'database'))
         if username:
             self.entry_login.set_text(username)
-            focus_widget = self.entry_password
         else:
             self.entry_login.set_text('')
-            focus_widget = self.entry_login
-        if not self.move_active:
-            focus_widget.grab_focus()
-        self.move_active = False
-
-    def profile_move_active(self, combobox, scrolltype):
-        self.move_active = True
 
     def clear_profile_combo(self, entry, event):
-        host_entry = self.entry_host.get_text()
-        host, port = host_entry.split(':', 1) if ':' in host_entry else ('', '')
+        netloc = self.entry_host.get_text()
+        host = self.get_hostname(netloc)
+        try:
+            port = str(self.get_port(netloc))
+        except ValueError:
+            host = ''
+            port = ''
         database = self.entry_database.get_text().strip()
         for idx, profile_info in enumerate(self.profile_store):
+            if not profile_info[1]:
+                continue
             profile = profile_info[0]
             if (host == self.profiles.get(profile, 'host')
                     and port == self.profiles.get(profile, 'port')
@@ -522,6 +552,7 @@ class DBLogin(object):
         else:
             idx = -1
         self.combo_profile.set_active(idx)
+        return False
 
     def expand_hostspec(self, expander, *args):
         visibility = expander.props.expanded
@@ -530,7 +561,22 @@ class DBLogin(object):
         self.entry_database.props.visible = visibility
         self.label_database.props.visible = visibility
 
-    def run(self, parent):
+    def get_hostname(self, netloc):
+        if '[' in netloc and ']' in netloc:
+            return netloc.split(']')[0][1:]
+        elif ':' in netloc:
+            return netloc.split(':')[0]
+        else:
+            return netloc
+
+    def get_port(self, netloc):
+        netloc = netloc.split(']')[-1]
+        if ':' in netloc:
+            return int(netloc.split(':')[1])
+        else:
+            return 8000
+
+    def run(self):
         profile_name = CONFIG['login.profile']
         can_use_profile = self.profiles.has_section(profile_name)
         if can_use_profile:
@@ -548,7 +594,11 @@ class DBLogin(object):
                     break
         else:
             self.combo_profile.set_active(-1)
-            self.entry_host.set_text('%s:%s' % (CONFIG['login.server'],
+            if ':' in CONFIG['login.server']:
+                host = '[%s]' % CONFIG['login.server']
+            else:
+                host = CONFIG['login.server']
+            self.entry_host.set_text('%s:%s' % (host,
                 CONFIG['login.port']))
             db = CONFIG['login.db'] if CONFIG['login.db'] else ''
             self.entry_database.set_text(db)
@@ -567,15 +617,31 @@ class DBLogin(object):
         self.expand_hostspec(self.expander)
 
         res, result = None, ('', '', '', '', '')
-        while not (res in (gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT)
-            or (res == gtk.RESPONSE_OK and all(result))):
+        while not all(result):
             res = self.dialog.run()
+            if res != gtk.RESPONSE_OK:
+                break
             active_profile = self.combo_profile.get_active()
             if active_profile != -1:
                 profile = self.profile_store[active_profile][0]
                 CONFIG['login.profile'] = profile
-            host, port = (self.entry_host.get_text().split(':', 1)
-                + ['8070'])[:2]
+            netloc = self.entry_host.get_text()
+            host = self.get_hostname(netloc)
+            try:
+                port = self.get_port(netloc)
+            except ValueError:
+                continue
+            if ':' in host:
+                hostname = '[%s]' % host
+            else:
+                hostname = host
+            try:
+                if not common.test_server_version(host, port):
+                    common.warning('', _(u'Incompatible version of the server'))
+                    continue
+            except Exception, exception:
+                common.process_exception(exception)
+                continue
             database = self.entry_database.get_text()
             login = self.entry_login.get_text()
             CONFIG['login.server'] = host
@@ -584,16 +650,14 @@ class DBLogin(object):
             CONFIG['login.expanded'] = self.expander.props.expanded
             CONFIG['login.login'] = login
             result = (self.entry_login.get_text(),
-                self.entry_password.get_text(), host, int(port), database)
+                self.entry_password.get_text(), host, port, database)
 
+        self.parent.present()
+        self.dialog.destroy()
         if res != gtk.RESPONSE_OK:
-            parent.present()
-            self.dialog.destroy()
             rpc.logout()
             from tryton.gui.main import Main
             Main.get_main().refresh_ssl()
-            raise Exception('QueryCanceled')
-        parent.present()
-        self.dialog.destroy()
+            raise TrytonError('QueryCanceled')
         return result
 
