@@ -1,15 +1,17 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+import os
+import tempfile
 import tryton.rpc as rpc
 from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT, datetime_strftime, \
         domain_inversion, eval_domain, localize_domain, unlocalize_domain, \
-        merge, inverse_leaf, EvalEnvironment
+        merge, inverse_leaf, EvalEnvironment, RPCProgress
 import tryton.common as common
-from tryton.pyson import PYSONDecoder
 import time
 import datetime
 from decimal import Decimal
 import logging
+from tryton.exceptions import TrytonServerError
 
 
 class Field(object):
@@ -42,7 +44,7 @@ class CharField(object):
         record.autocomplete_with(self.name)
 
     def domains_get(self, record):
-        screen_domain = domain_inversion(record.group.domain,
+        screen_domain = domain_inversion(record.group.domain4inversion,
             self.name, EvalEnvironment(record, False))
         if isinstance(screen_domain, bool) and not screen_domain:
             screen_domain = [('id', '=', False)]
@@ -138,12 +140,7 @@ class CharField(object):
         if (internal or False) != (record.value.get(self.name, False) or False):
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
-            try:
-                self.sig_changed(record)
-            except Exception:
-                record.value[self.name] = internal
-                record.modified_fields = prev_modified_fields
-                return
+            self.sig_changed(record)
             record.validate(softvalidation=True)
             record.signal('record-changed')
 
@@ -241,12 +238,7 @@ class FloatField(CharField):
             if not self.get_state_attrs(record).get('readonly', False):
                 record.modified_fields.setdefault(self.name)
                 record.signal('record-modified')
-                try:
-                    self.sig_changed(record)
-                except Exception:
-                    record.value[self.name] = internal
-                    record.modified_fields = prev_modified_fields
-                    return
+                self.sig_changed(record)
                 record.validate(softvalidation=True)
                 record.signal('record-changed')
 
@@ -265,12 +257,7 @@ class NumericField(CharField):
             if not self.get_state_attrs(record).get('readonly', False):
                 record.modified_fields.setdefault(self.name)
                 record.signal('record-modified')
-                try:
-                    self.sig_changed(record)
-                except Exception:
-                    record.value[self.name] = internal
-                    record.prev_modified_fields = prev_modified_fields
-                    return
+                self.sig_changed(record)
                 record.validate(softvalidation=True)
                 record.signal('record-changed')
 
@@ -293,12 +280,7 @@ class BooleanField(CharField):
         if internal != bool(record.value.get(self.name, False)):
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
-            try:
-                self.sig_changed(record)
-            except Exception:
-                record.value[self.name] = internal
-                record.modified_fields = prev_modified_fields
-                return
+            self.sig_changed(record)
             record.validate(softvalidation=True)
             record.signal('record-changed')
 
@@ -316,12 +298,13 @@ class M2OField(CharField):
 
     def get(self, record, check_load=True, readonly=True, modified=False):
         value = record.value.get(self.name)
-        if record.parent_name == self.name:
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
             value = record.parent.id if record.parent else False
         if value:
             if isinstance(value, (int, basestring, long)):
                 self.set(record, value)
-                value = record.value.get(self.name)
+                value = record.value.get(self.name, value)
             if isinstance(value, (int, basestring, long)):
                 return value
             return value[0] or False
@@ -329,19 +312,21 @@ class M2OField(CharField):
 
     def get_client(self, record):
         value = record.value.get(self.name)
-        if record.parent_name == self.name:
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
             value = record.parent.id if record.parent else False
         if value:
             if isinstance(value, (int, basestring, long)):
                 self.set(record, value)
-                value = record.value.get(self.name)
+                value = record.value.get(self.name, value)
             if isinstance(value, (int, basestring, long)):
                 return value
             return value[1]
         return False
 
     def set(self, record, value, modified=False):
-        if record.parent_name == self.name:
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
             if record.parent:
                 if 'rec_name' in record.parent.value:
                     value = (record.parent.id, record.parent.value['rec_name'])
@@ -349,14 +334,13 @@ class M2OField(CharField):
                     value = record.parent.id
             else:
                 value = False
-        if value and isinstance(value, (int, long)) and value > 0:
+        if value and isinstance(value, (int, long)) and value >= 0:
             args = ('model', self.attrs['relation'], 'read', value,
                     ['rec_name'], rpc.CONTEXT)
             try:
                 result = rpc.execute(*args)
-            except Exception, exception:
-                result = common.process_exception(exception, record.window,
-                        *args)
+            except TrytonServerError, exception:
+                result = common.process_exception(exception, *args)
                 if not result:
                     return
             value = value, result['rec_name']
@@ -370,7 +354,8 @@ class M2OField(CharField):
             else:
                 record.value[self.name + '.rec_name'] = ''
         record.value[self.name] = value or (False, '')
-        if record.parent_name == self.name:
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
             if record.parent:
                 if 'rec_name' not in record.parent.value:
                     record.parent.value['rec_name'] = \
@@ -387,20 +372,11 @@ class M2OField(CharField):
         if (internal[0] or False) != (record.value[self.name][0] or False):
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
-            try:
-                self.sig_changed(record)
-            except Exception:
-                record.value[self.name] = internal
-                record.modified_fields = prev_modified_fields
-                return
+            self.sig_changed(record)
             record.validate(softvalidation=True)
             record.signal('record-changed')
         elif force_change:
-            try:
-                self.sig_changed(record)
-            except Exception:
-                record.value[self.name] = internal
-                return
+            self.sig_changed(record)
             record.validate(softvalidation=True)
             record.signal('record-changed')
 
@@ -422,7 +398,8 @@ class M2OField(CharField):
 
     def get_state_attrs(self, record):
         result = super(M2OField, self).get_state_attrs(record)
-        if record.parent_name == self.name:
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
             result = result.copy()
             result['readonly'] = True
         return result
@@ -455,17 +432,19 @@ class O2MField(CharField):
         record.parent.signal('record-changed')
 
     def _group_list_changed(self, group, signal):
-        group.parent.group.signal('group-list-changed', signal)
+        if group.model_name == group.parent.model_name:
+            group.parent.group.signal('group-list-changed', signal)
 
     def _group_cleared(self, group, signal):
-        group.parent.signal('group-cleared')
+        if group.model_name == group.parent.model_name:
+            group.parent.signal('group-cleared')
 
     def _set_default_value(self, record):
         if record.value.get(self.name) is not None:
             return
         from group import Group
         parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], {}, record.window,
+        group = Group(self.attrs['relation'], {},
                 parent=record,
                 parent_name=parent_name,
                 child_name=self.name,
@@ -485,23 +464,29 @@ class O2MField(CharField):
     def get(self, record, check_load=True, readonly=True, modified=False):
         if record.value.get(self.name) is None:
             return []
+        record_removed = record.value[self.name].record_removed
+        record_deleted = record.value[self.name].record_deleted
         result = [('add', [])]
+        parent_name = self.attrs.get('relation_field', '')
         for record2 in record.value[self.name]:
-            if record2.id > 0:
+            if record2 in record_removed or record2 in record_deleted:
+                continue
+            if record2.id >= 0:
                 values = record2.get(check_load=check_load,
                     get_readonly=readonly, get_modifiedonly=modified)
+                values.pop(parent_name, None)
                 if record2.modified and values:
                     result.append(('write', record2.id, values))
                 result[0][1].append(record2.id)
             else:
-                result.append(('create',
-                    record2.get(check_load=check_load, get_readonly=readonly)))
-        if record.value[self.name].record_removed:
-            result.append(('unlink', [x.id for x in \
-                record.value[self.name].record_removed]))
-        if record.value[self.name].record_deleted:
-            result.append(('delete', [x.id for x in \
-                record.value[self.name].record_deleted]))
+                values = record2.get(check_load=check_load,
+                    get_readonly=readonly)
+                values.pop(parent_name, None)
+                result.append(('create', values))
+        if record_removed:
+            result.append(('unlink', [x.id for x in record_removed]))
+        if record_deleted:
+            result.append(('delete', [x.id for x in record_deleted]))
         return result
 
     def get_timestamp(self, record):
@@ -536,7 +521,7 @@ class O2MField(CharField):
         elif record.model_name == self.attrs['relation']:
             fields = record.group.fields
         parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], {}, record.window,
+        group = Group(self.attrs['relation'], {},
                 parent=record, parent_name=parent_name,
                 child_name=self.name,
                 context=self.context,
@@ -569,9 +554,9 @@ class O2MField(CharField):
             fields = group.fields
         elif record.model_name == self.attrs['relation']:
             fields = record.group.fields
-
-        # value is a list of dict
-        fields_dict = {}
+        if fields:
+            fields = dict((fname, field.attrs)
+                for fname, field in fields.iteritems())
         if value and len(value):
             context = self.context_get(record)
             field_names = []
@@ -584,25 +569,23 @@ class O2MField(CharField):
                 args = ('model', self.attrs['relation'], 'fields_get',
                         field_names, context)
                 try:
-                    fields_to_load = rpc.execute(*args)
-                except Exception, exception:
-                    fields_dict = common.process_exception(exception,
-                            record.window, *args)
-                    if not fields_dict:
+                    fields.update(rpc.execute(*args))
+                except TrytonServerError, exception:
+                    fields = common.process_exception(exception, *args)
+                    if not fields:
                         return False
 
         parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], fields_dict, record.window,
+        group = Group(self.attrs['relation'], fields,
                 parent=record, parent_name=parent_name, child_name=self.name,
                 context=self.context,
                 parent_datetime_field=self.attrs.get('datetime_field'))
         if record.value.get(self.name):
             group.record_deleted.extend(x for x in record.value[self.name]
-                if x.id > 0)
+                if x.id >= 0)
             group.record_deleted.extend(record.value[self.name].record_deleted)
             group.record_removed.extend(record.value[self.name].record_removed)
         record.value[self.name] = group
-        group.fields = fields
         for vals in (value or []):
             new_record = record.value[self.name].new(default=False)
             new_record.set_default(vals, modified=modified)
@@ -628,9 +611,8 @@ class O2MField(CharField):
                     field_names, context)
             try:
                 fields = rpc.execute(*args)
-            except Exception, exception:
-                fields = common.process_exception(exception, record.window,
-                        *args)
+            except TrytonServerError, exception:
+                fields = common.process_exception(exception, *args)
                 if not fields:
                     return False
 
@@ -707,7 +689,7 @@ class O2MField(CharField):
 class M2MField(O2MField):
 
     def get_default(self, record):
-        return [x.id for x in record.value.get(self.name) or [] if x.id > 0]
+        return [x.id for x in record.value.get(self.name) or [] if x.id >= 0]
 
     def get_eval(self, record, check_load=True):
         return [x.id for x in record.value.get(self.name) or []]
@@ -722,7 +704,7 @@ class M2MField(O2MField):
         elif record.model_name == self.attrs['relation']:
             fields = record.group.fields
         parent_name = self.attrs.get('relation_field', '')
-        group = Group(self.attrs['relation'], {}, record.window,
+        group = Group(self.attrs['relation'], {},
                 parent=record, parent_name=parent_name,
                 child_name=self.name,
                 context=self.context,
@@ -767,12 +749,7 @@ class ReferenceField(CharField):
         if (internal or False) != (record.value[self.name] or False):
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
-            try:
-                self.sig_changed(record)
-            except Exception:
-                record.value[self.name] = internal
-                record.modified_fields = prev_modified_fields
-                return
+            self.sig_changed(record)
             record.validate(softvalidation=True)
             record.signal('record-changed')
 
@@ -786,16 +763,15 @@ class ReferenceField(CharField):
         ref_model, (ref_id, ref_str) = value
         if ref_model:
             ref_id = int(ref_id)
-            if not ref_id:
+            if ref_id < 0:
                 ref_str = ''
-            if not ref_str and ref_id > 0:
+            if not ref_str and ref_id >= 0:
                 args = ('model', ref_model, 'read', ref_id,
                         ['rec_name'], rpc.CONTEXT)
                 try:
                     result = rpc.execute(*args)
-                except Exception, exception:
-                    result = common.process_exception(exception, record.window,
-                            *args)
+                except TrytonServerError, exception:
+                    result = common.process_exception(exception, *args)
                     if not result:
                         return
                 result = result['rec_name']
@@ -803,7 +779,7 @@ class ReferenceField(CharField):
                     record.value[self.name] = ref_model, (ref_id, result)
                     record.value[self.name + '.rec_name'] = result
                 else:
-                    record.value[self.name] = ref_model, (0, '')
+                    record.value[self.name] = ref_model, (-1, '')
                     record.value[self.name + '.rec_name'] = ''
             else:
                 record.value[self.name] = ref_model, (ref_id, ref_str)
@@ -816,6 +792,59 @@ class ReferenceField(CharField):
             record.modified_fields.setdefault(self.name)
             record.signal('record-modified')
 
+class BinaryField(CharField):
+
+    def get(self, record, check_load=True, readonly=True, modified=False):
+        result = record.value.get(self.name, False) or False
+        if isinstance(result, basestring):
+            try:
+                with open(result, 'rb') as fp:
+                    result = buffer(fp.read())
+            except IOError:
+                result = self.get_data(record)
+        return result
+
+    def get_client(self, record):
+        return self.get(record)
+
+    def set_client(self, record, value, force_change=False):
+        _, filename = tempfile.mkstemp(prefix='tryton_')
+        with open(filename, 'wb') as fp:
+            fp.write(value or '')
+        self.set(record, filename)
+        record.modified_fields.setdefault(self.name)
+        record.signal('record-modified')
+        self.sig_changed(record)
+        record.validate(softvalidation=True)
+        record.signal('record-changed')
+
+    def get_size(self, record):
+        result = record.value.get(self.name) or 0
+        if isinstance(result, basestring):
+            result = os.stat(result).st_size
+        elif isinstance(result, buffer):
+            result = len(result)
+        return result
+
+    def get_data(self, record):
+        if not isinstance(record.value.get(self.name), (basestring, buffer)):
+            if record.id < 0:
+                return ''
+            ctx = rpc.CONTEXT.copy()
+            ctx.update(record.context_get())
+            rpcprogress = RPCProgress('execute', ('model', record.model_name,
+                    'read', record.id, [self.name], ctx))
+            try:
+                values = rpcprogress.run()
+            except TrytonServerError, exception:
+                values = common.process_exception(exception)
+                return ''
+            _, filename = tempfile.mkstemp(prefix='tryton_')
+            with open(filename, 'wb') as fp:
+                fp.write(values[self.name] or '')
+            self.set(record, filename)
+        return self.get(record)
+
 TYPES = {
     'char': CharField,
     'sha': CharField,
@@ -833,4 +862,5 @@ TYPES = {
     'datetime': DateTimeField,
     'date': DateField,
     'one2one': O2OField,
+    'binary': BinaryField,
 }
