@@ -8,6 +8,7 @@ from tryton.gui.window.view_form.screen import Screen
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
 from tryton.config import CONFIG
+from tryton.exceptions import TrytonServerError
 import tryton.common as common
 import tryton.rpc as rpc
 import pango
@@ -17,9 +18,8 @@ _ = gettext.gettext
 
 class One2Many(WidgetInterface):
 
-    def __init__(self, field_name, model_name, window, attrs=None):
-        super(One2Many, self).__init__(field_name, model_name, window,
-                attrs=attrs)
+    def __init__(self, field_name, model_name, attrs=None):
+        super(One2Many, self).__init__(field_name, model_name, attrs=attrs)
 
         self.widget = gtk.VBox(homogeneous=False, spacing=2)
         self._readonly = True
@@ -40,6 +40,7 @@ class One2Many(WidgetInterface):
             self.wid_text = gtk.Entry()
             self.wid_text.set_property('width_chars', 13)
             self.wid_text.connect('activate', self._sig_activate)
+            self.wid_text.connect('focus-out-event', self._focus_out)
             hbox.pack_start(self.wid_text, expand=True, fill=True)
 
             self.but_add = gtk.Button()
@@ -153,15 +154,13 @@ class One2Many(WidgetInterface):
         frame.set_shadow_type(gtk.SHADOW_OUT)
         self.widget.pack_start(frame, expand=False, fill=True)
 
-        self.screen = Screen(attrs['relation'], self.window,
-                mode=attrs.get('mode', 'tree,form').split(','),
-                views_preload=attrs.get('views', {}),
-                row_activate=self._on_activate,
-                exclude_field=attrs.get('relation_field', None))
+        self.screen = Screen(attrs['relation'],
+            mode=attrs.get('mode', 'tree,form').split(','),
+            views_preload=attrs.get('views', {}),
+            row_activate=self._on_activate,
+            exclude_field=attrs.get('relation_field', None))
         self.screen.signal_connect(self, 'record-message', self._sig_label)
 
-        if not isinstance(self.screen.window, gtk.Dialog):
-            self.screen.widget.set_size_request(0, 0)
         self.widget.pack_start(self.screen.widget, expand=True, fill=True)
 
         self.screen.widget.connect('key_press_event', self.on_keypress)
@@ -235,17 +234,19 @@ class One2Many(WidgetInterface):
         sequence = None
         if self.screen.current_view.view_type == 'tree':
             sequence = self.screen.current_view.widget_tree.sequence
+
+        def update_sequence():
+            if sequence:
+                self.screen.group.set_sequence(field=sequence)
+
         if (self.screen.current_view.view_type == 'form') \
                 or self.screen.editable_get():
             self.screen.new(context=ctx)
             self.screen.current_view.widget.set_sensitive(True)
+            update_sequence()
         else:
-            win = WinForm(self.screen, self.window, new=True,
+            WinForm(self.screen, lambda a: update_sequence(), new=True,
                 many=True, context=ctx)
-            win.run()
-            win.destroy()
-        if sequence:
-            self.screen.group.set_sequence(field=sequence)
 
     def _sig_edit(self, widget=None):
         self.view.set_value()
@@ -255,9 +256,7 @@ class One2Many(WidgetInterface):
             if not record.validate(fields):
                 self.screen.display()
                 return
-            win = WinForm(self.screen, self.window)
-            win.run()
-            win.destroy()
+            WinForm(self.screen, lambda a: None)
 
     def _sig_next(self, widget):
         self.view.set_value()
@@ -289,6 +288,10 @@ class One2Many(WidgetInterface):
         self._sig_add()
         self.wid_text.grab_focus()
 
+    def _focus_out(self, *args):
+        if self.wid_text.get_text():
+            self._sig_add()
+
     def _sig_add(self, *args):
         self.view.set_value()
         domain = self.field.domain_get(self.record)
@@ -307,23 +310,25 @@ class One2Many(WidgetInterface):
                 dom = ['OR', domain, ('id', 'in', removed_ids)]
             ids = rpc.execute('model', self.attrs['relation'], 'search', dom,
                     0, CONFIG['client.limit'], None, context)
-        except Exception, exception:
-            common.process_exception(exception, self.window)
+        except TrytonServerError, exception:
+            common.process_exception(exception)
             return False
+        def callback(ids):
+            res_id = None
+            if ids:
+                res_id = ids[0]
+            self.screen.load(ids, modified=True)
+            self.screen.display(res_id=res_id)
+            if self.screen.current_view:
+                self.screen.current_view.set_cursor()
+            self.wid_text.set_text('')
         if len(ids) != 1:
-            win = WinSearch(self.attrs['relation'], sel_multi=True, ids=ids,
-                    context=context, domain=domain, parent=self.window,
-                    views_preload=self.attrs.get('views', {}))
-            ids = win.run()
+            WinSearch(self.attrs['relation'], callback, sel_multi=True,
+                ids=ids, context=context, domain=domain,
+                views_preload=self.attrs.get('views', {}))
+        else:
+            callback(ids)
 
-        res_id = None
-        if ids:
-            res_id = ids[0]
-        self.screen.load(ids, modified=True)
-        self.screen.display(res_id=res_id)
-        if self.screen.current_view:
-            self.screen.current_view.set_cursor()
-        self.wid_text.set_text('')
 
     def _sig_label(self, screen, signal_data):
         name = '_'
@@ -379,10 +384,8 @@ class One2Many(WidgetInterface):
         self.screen.display()
         return True
 
-    def display_value(self):
-        return '<' + self.attrs.get('string', '') + '>'
-
     def set_value(self, record, field):
+        self.screen.save_tree_state()
         self.screen.current_view.set_value()
         if self.screen.modified(): # TODO check if required
             record.modified_fields.setdefault(field.name)
