@@ -1,6 +1,7 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 
+import gobject
 from editabletree import EditableTreeView
 from tryton.gui.window.view_form.view.interface import ParserInterface
 from tryton.gui.window.win_search import WinSearch
@@ -11,6 +12,7 @@ import tryton.rpc as rpc
 from tryton.common import DT_FORMAT, DHM_FORMAT, COLORS, node_attributes, \
         TRYTON_ICON, HM_FORMAT
 import tryton.common as common
+from tryton.exceptions import TrytonError, TrytonServerError
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrendererdate import CellRendererDate
 from tryton.common.cellrenderertext import CellRendererText
@@ -26,6 +28,7 @@ import locale
 import datetime
 import time
 import gettext
+import operator
 
 _ = gettext.gettext
 
@@ -33,7 +36,7 @@ def send_keys(renderer, editable, position, treeview):
     editable.connect('key_press_event', treeview.on_keypressed)
     editable.editing_done_id = editable.connect('editing_done',
             treeview.on_editing_done)
-    if isinstance(editable, gtk.ComboBoxEntry):
+    if isinstance(editable, (gtk.ComboBoxEntry, gtk.ComboBox)):
         editable.connect('changed', treeview.on_editing_done)
 
 def sort_model(column, treeview, screen):
@@ -56,19 +59,20 @@ def sort_model(column, treeview, screen):
             column.arrow.hide()
     store = treeview.get_model()
     unsaved_records = [x for x in store.group if x.id < 0]
+    search_string = screen.screen_container.get_text() or None
     if screen.search_count == len(store):
-        ids = screen.search_filter(only_ids=True)
+        ids = screen.search_filter(search_string=search_string, only_ids=True)
         store.sort(ids)
     else:
-        screen.search_filter()
+        screen.search_filter(search_string=search_string)
     for record in unsaved_records:
         store.group.append(record)
 
 class ParserTree(ParserInterface):
 
-    def __init__(self, window, parent=None, attrs=None, screen=None,
+    def __init__(self, parent=None, attrs=None, screen=None,
             children_field=None):
-        super(ParserTree, self).__init__(window, parent, attrs, screen,
+        super(ParserTree, self).__init__(parent, attrs, screen,
             children_field=children_field)
         self.treeview = None
 
@@ -110,7 +114,7 @@ class ParserTree(ParserInterface):
                         node_attrs[attr_name] = fields[fname].attrs[attr_name]
                 cell = CELLTYPES.get(node_attrs.get('widget',
                     fields[fname].attrs['type']))(fname, model_name,
-                    treeview, self.window, node_attrs)
+                    treeview, node_attrs)
                 treeview.cells[fname] = cell
                 renderer = cell.renderer
 
@@ -185,6 +189,7 @@ class ParserTree(ParserInterface):
                 col.width = width
                 if width > 0:
                     col.set_fixed_width(width)
+                col.set_min_width(1)
                 #XXX doesn't work well when resize columns
                 #col.set_expand(True)
                 if (not treeview.sequence
@@ -203,14 +208,15 @@ class ParserTree(ParserInterface):
                                 'float_time'):
                     label = gtk.Label(node_attrs['sum'] + _(': '))
                     label_sum = gtk.Label()
-                    if isinstance(fields[fname].attrs.get('digits'), str):
+                    if isinstance(fields[fname].attrs.get('digits'),
+                            basestring):
                         digits = 2
                     else:
                         digits = fields[fname].attrs.get('digits', (16, 2))[1]
                     dict_widget[i] = (fname, label, label_sum, digits)
             elif node.localName == 'button':
                 #TODO add shortcut
-                cell = Button(treeview, self.window, self.screen, node_attrs)
+                cell = Button(treeview, self.screen, node_attrs)
                 button_list.append(cell)
                 renderer = cell.renderer
                 string = node_attrs.get('string', _('Unknown'))
@@ -255,14 +261,13 @@ class ParserTree(ParserInterface):
 
 class Char(object):
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Char, self).__init__()
         self.field_name = field_name
         self.model_name = model_name
         self.attrs = attrs or {}
         self.renderer = CellRendererText()
         self.treeview = treeview
-        self.window = window
 
     def setter(self, column, cell, store, iter):
         record = store.get_value(iter, 0)
@@ -334,7 +339,8 @@ class Char(object):
     def get_color(self, record):
         return record.expr_eval(self.treeview.colors, check_load=False)
 
-    def open_remote(self, record, create, changed=False, text=None):
+    def open_remote(self, record, create, changed=False, text=None,
+            callback=None):
         raise NotImplementedError
 
     def get_textual_value(self, record):
@@ -342,18 +348,24 @@ class Char(object):
             return ''
         return record[self.field_name].get_client(record) or ''
 
-    def value_from_text(self, record, text):
-        return text
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        field.set_client(record, text)
+        if callback:
+            callback()
 
 class Int(Char):
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
-        super(Int, self).__init__(field_name, model_name, treeview, window,
-                attrs=attrs)
+    def __init__(self, field_name, model_name, treeview, attrs=None):
+        super(Int, self).__init__(field_name, model_name, treeview,
+            attrs=attrs)
         self.renderer = CellRendererInteger()
 
-    def value_from_text(self, record, text):
-        return int(text)
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        field.set_client(record, int(text))
+        if callback:
+            callback()
 
     def get_textual_value(self, record):
         return locale.format('%d',
@@ -361,9 +373,9 @@ class Int(Char):
 
 class Boolean(Int):
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
-        super(Boolean, self).__init__(field_name, model_name, treeview, window,
-                attrs=attrs)
+    def __init__(self, field_name, model_name, treeview, attrs=None):
+        super(Boolean, self).__init__(field_name, model_name, treeview,
+            attrs=attrs)
         self.renderer = CellRendererToggle()
         self.renderer.connect('toggled', self._sig_toggled)
 
@@ -384,9 +396,9 @@ class Boolean(Int):
 class Date(Char):
     server_format = DT_FORMAT
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
-        super(Date, self).__init__(field_name, model_name, treeview, window,
-                attrs=attrs)
+    def __init__(self, field_name, model_name, treeview, attrs=None):
+        super(Date, self).__init__(field_name, model_name, treeview,
+            attrs=attrs)
         self.display_format = date_format()
         self.renderer = CellRendererDate(self.display_format)
 
@@ -397,22 +409,25 @@ class Date(Char):
         date = datetime.date(*time.strptime(value, self.server_format)[:3])
         return common.datetime_strftime(date, self.display_format)
 
-    def value_from_text(self, record, text):
-        if not text:
-            return False
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        date = False
         try:
             date = datetime.date(*time.strptime(text, self.display_format)[:3])
-        except Exception:
-            return False
-        return common.datetime_strftime(date, self.server_format)
+            date = common.datetime_strftime(date, self.server_format)
+        except ValueError:
+            date = False
+        field.set_client(record, date)
+        if callback:
+            callback()
 
 
 class Datetime(Date):
     server_format = DHM_FORMAT
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Datetime, self).__init__(field_name, model_name, treeview,
-                window, attrs=attrs)
+            attrs=attrs)
         self.display_format = date_format() + ' ' + HM_FORMAT
         self.renderer.format = self.display_format
 
@@ -422,77 +437,60 @@ class Datetime(Date):
             return ''
         date = datetime.datetime(*time.strptime(value,
             self.server_format)[:6])
-        if 'timezone' in rpc.CONTEXT:
-            try:
-                import pytz
-                lzone = pytz.timezone(rpc.CONTEXT['timezone'])
-                szone = pytz.timezone(rpc.TIMEZONE)
-                sdt = szone.localize(date, is_dst=True)
-                ldt = sdt.astimezone(lzone)
-                date = ldt
-            except Exception:
-                pass
+        date = common.timezoned_date(date)
         return common.datetime_strftime(date, self.display_format)
 
-    def value_from_text(self, record, text):
-        if not text:
-            return False
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        date = False
         try:
             date = datetime.datetime(*time.strptime(text,
                 self.display_format)[:6])
-        except Exception:
-            return False
-        if 'timezone' in rpc.CONTEXT:
-            try:
-                import pytz
-                lzone = pytz.timezone(rpc.CONTEXT['timezone'])
-                szone = pytz.timezone(rpc.TIMEZONE)
-                ldt = lzone.localize(date, is_dst=True)
-                sdt = ldt.astimezone(szone)
-                date = sdt
-            except Exception:
-                pass
-        return common.datetime_strftime(date, self.server_format)
+            date = common.timezoned_date(date)
+            date = common.datetime_strftime(date, self.server_format)
+        except ValueError:
+            date = False
+        field.set_client(record, date)
+        if callback:
+            callback()
 
 
 class Float(Char):
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
-        super(Float, self).__init__(field_name, model_name, treeview, window,
-                attrs=attrs)
+    def __init__(self, field_name, model_name, treeview, attrs=None):
+        super(Float, self).__init__(field_name, model_name, treeview,
+            attrs=attrs)
         self.renderer = CellRendererFloat()
 
     def setter(self, column, cell, store, iter):
         super(Float, self).setter(column, cell, store, iter)
         record = store.get_value(iter, 0)
         field = record[self.field_name]
-        if isinstance(field.attrs.get('digits'), str):
-            digits = record.expr_eval(field.attrs['digits'])
-        else:
-            digits = field.attrs.get('digits', (16, 2))
+        digits = record.expr_eval(field.attrs.get('digits', (16, 2)))
         cell.digits = digits
 
     def get_textual_value(self, record):
         field = record[self.field_name]
-        if isinstance(field.attrs.get('digits'), str):
-            digit = record.expr_eval(field.attrs['digits'])[1]
-        else:
-            digit = field.attrs.get('digits', (16, 2))[1]
+        digit = record.expr_eval(field.attrs.get('digits', (16, 2)))[1]
         return locale.format('%.'+str(digit)+'f',
                 record[self.field_name].get_client(record) or 0.0, True)
 
-    def value_from_text(self, record, text):
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
         try:
-            return locale.atof(text)
-        except Exception:
-            return 0.0
+            value = locale.atof(text)
+        except ValueError:
+            value = 0.0
+        field.set_client(record, value)
+        if callback:
+            callback()
 
 
 class FloatTime(Char):
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None):
         super(FloatTime, self).__init__(field_name, model_name, treeview,
-                window, attrs=attrs)
+            attrs=attrs)
         self.conv = None
         if attrs and attrs.get('float_time'):
             self.conv = rpc.CONTEXT.get(attrs['float_time'])
@@ -501,15 +499,20 @@ class FloatTime(Char):
         val = record[self.field_name].get_client(record)
         return common.float_time_to_text(val, self.conv)
 
-    def value_from_text(self, record, text):
-        return common.text_to_float_time(text, self.conv)
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        field.set_client(record, common.text_to_float_time(text, self.conv))
+        if callback:
+            callback()
 
 class M2O(Char):
 
-    def value_from_text(self, record, text):
+    def value_from_text(self, record, text, callback=None):
         field = record.group.fields[self.field_name]
         if not text and not field.get_state_attrs(
                 record)['required']:
+            if callback:
+                callback()
             return False
 
         relation = record[self.field_name].attrs['relation']
@@ -524,16 +527,24 @@ class M2O(Char):
                 context)
         try:
             ids = rpc.execute(*args)
-        except Exception, exception:
-            ids = common.process_exception(exception, self.window, *args)
+        except TrytonServerError, exception:
+            ids = common.process_exception(exception, *args)
             if not ids:
-                return '???'
+                field.set_client(record, '???')
+                if callback:
+                    callback()
+                return
         if len(ids) != 1:
-            return self.search_remote(relation, ids,
-                             domain=domain, context=context)[0]
-        return ids[0]
+            if callback:
+                self.search_remote(record, relation, ids, domain=domain,
+                    context=context, callback=callback)
+            return
+        field.set_client(record, ids[0])
+        if callback:
+            callback()
 
-    def open_remote(self, record, create=True, changed=False, text=None):
+    def open_remote(self, record, create=True, changed=False, text=None,
+            callback=None):
         field = record.group.fields[self.field_name]
         relation = field.attrs['relation']
 
@@ -551,55 +562,62 @@ class M2O(Char):
             args = ('model', relation, 'search', dom, 0, None, None, context)
             try:
                 ids = rpc.execute(*args)
-            except Exception, exception:
-                ids = common.process_exception(exception, self.window, *args)
+            except TrytonServerError, exception:
+                ids = common.process_exception(exception, *args)
                 if not ids:
-                    return False, False
+                    field.set_client(record, False)
+                    if callback:
+                        callback()
+                    return
             if len(ids) == 1:
-                return True, ids[0]
-            searched = self.search_remote(relation, ids, domain=domain,
-                    context=context)
-            if searched[0]:
-                return True, searched
-            return False, False
-        screen = Screen(relation, self.window, domain=domain, context=context,
-                mode=['form'])
+                field.set_client(record, ids[0])
+                if callback:
+                    callback()
+                return
+            self.search_remote(record, relation, ids, domain=domain,
+                context=context, callback=callback)
+            return
+        screen = Screen(relation, domain=domain, context=context,
+            mode=['form'])
+
+        def open_callback(result):
+            if result and screen.save_current():
+                value = (screen.current_record.id,
+                    screen.current_record.rec_name())
+                field.set_client(record, value)
+            if callback:
+                callback()
         if obj_id:
             screen.load([obj_id])
-            win = WinForm(screen, self.window)
+            WinForm(screen, open_callback)
         else:
-            win = WinForm(screen, self.window, new=True)
-        value = False
-        if win.run():
-            if screen.save_current():
-                value = (screen.current_record.id,
-                        screen.current_record.rec_name())
-        win.destroy()
-        if value:
-            return True, value
-        else:
-            return False, False
+            WinForm(screen, open_callback, new=True)
+
+    def search_remote(self, record, relation, ids=None, domain=None,
+            context=None, callback=None):
+        field = record.group.fields[self.field_name]
+        def search_callback(found):
+            value = None
+            if found:
+                args = ('model', relation, 'read', found[0], ['rec_name'],
+                        context)
+                try:
+                    res = rpc.execute(*args)
+                except TrytonServerError, exception:
+                    res = common.process_exception(exception, *args)
+                if res:
+                    value = (found[0], res['rec_name'])
+            field.set_client(record, value)
+            if callback:
+                callback()
+        WinSearch(relation, search_callback, sel_multi=False, ids=ids,
+            context=context, domain=domain)
 
-    def search_remote(self, relation, ids=None, domain=None, context=None):
-        win = WinSearch(relation, sel_multi=False, ids=ids, context=context,
-                domain=domain, parent=self.window)
-        found = win.run()
-        if found:
-            args = ('model', relation, 'read', found[0], ['rec_name'],
-                    context)
-            try:
-                res = rpc.execute(*args)
-            except Exception, exception:
-                res = common.process_exception(exception, self.window, *args)
-                if not res:
-                    return False, None
-            return found[0], res['rec_name']
-        else:
-            return False, None
 
 class O2O(M2O):
     pass
 
+
 class UnsettableColumn(Exception):
 
     def __init__(self):
@@ -616,22 +634,24 @@ class O2M(Char):
         return '( ' + str(len(record[self.field_name].\
                 get_client(record))) + ' )'
 
-    def value_from_text(self, record, text):
-        pass
+    def value_from_text(self, record, text, callback=None):
+        if callback:
+            callback()
 
-    def open_remote(self, record, create=True, changed=False, text=None):
+    def open_remote(self, record, create=True, changed=False, text=None,
+            callback=None):
         group = record.value[self.field_name]
         field = record.group.fields[self.field_name]
         relation = field.attrs['relation']
         context = field.context_get(record)
 
-        screen = Screen(relation, self.window, mode=['tree', 'form'],
-                exclude_field=field.attrs.get('relation_field'))
+        screen = Screen(relation, mode=['tree', 'form'],
+            exclude_field=field.attrs.get('relation_field'))
         screen.group = group
-        win = WinForm(screen, self.window, view_type='tree', context=context)
-        win.run()
-        win.destroy()
-        return False, False
+        def open_callback(result):
+            if callback:
+                callback()
+        WinForm(screen, open_callback, view_type='tree', context=context)
 
 
 class M2M(Char):
@@ -644,12 +664,17 @@ class M2M(Char):
         return '( ' + str(len(record[self.field_name].\
                 get_client(record))) + ' )'
 
-    def value_from_text(self, record, text):
-        if not text:
-            return []
-        if not (text[0] != '('):
-            return record[self.field_name].get(record)
+    def value_from_text(self, record, text, callback=None):
         field = record[self.field_name]
+        if not text:
+            field.set_client(record, [])
+            if callback:
+                callback()
+            return
+        if not (text[0] != '('):
+            if callback:
+                callback()
+            return
         relation = field.attrs['relation']
         domain = field.domain_get(record)
         context = field.context_get(record)
@@ -662,16 +687,25 @@ class M2M(Char):
                 None, context)
         try:
             ids = rpc.execute(*args)
-        except Exception, exception:
-            ids = common.process_exception(exception, self.window, *args)
+        except TrytonServerError, exception:
+            ids = common.process_exception(exception, *args)
             if ids is False:
-                return []
-        win = WinSearch(relation, sel_multi=True, ids=ids, context=context,
-                domain=domain, parent=self.window)
-        found = win.run()
-        return found or []
+                field.set_client(record, [])
+                if callback:
+                    callback()
+                return
+        if not callback:
+            return
+        def winsearch_callback(result):
+            field.set_client(record, result or [])
+            if callback:
+                callback()
+        WinSearch(relation, winsearch_callback, sel_multi=True, ids=ids,
+            context=context, domain=domain)
+        return
 
-    def open_remote(self, record, create=True, changed=False, text=None):
+    def open_remote(self, record, create=True, changed=False, text=None,
+            callback=None):
         field = record[self.field_name]
         relation = field.attrs['relation']
         context = field.context_get(record)
@@ -682,134 +716,153 @@ class M2M(Char):
             args = ('model', relation, 'search', domain)
             try:
                 ids = rpc.execute(*args)
-            except Exception, exception:
-                ids = common.process_exception(exception, self.window, *args)
+            except TrytonServerError, exception:
+                ids = common.process_exception(exception, *args)
                 if ids is False:
-                    return False, None
+                    field.set_client(record, False)
+                    if callback:
+                        callback()
             if ids and len(ids)==1:
-                return True, ids
+                field.set_client(record, ids)
+                if callback:
+                    callback()
+                return
         else:
             ids = [x.id for x in field.get_client(record)]
-        win = WinSearch(relation, sel_multi=True, ids=ids, context=context,
-                domain=domain, parent=self.window)
-        found = win.run()
-        if found:
-            return True, found
-        else:
-            return False, None
+        def open_callback(result):
+            if result:
+                field.set_client(record, result)
+            if callback:
+                callback()
+        WinSearch(relation, open_callback, sel_multi=True, ids=ids, context=context,
+            domain=domain)
+
 
 class Selection(Char):
 
     def __init__(self, *args):
         super(Selection, self).__init__(*args)
         self.renderer = CellRendererCombo()
-        selection_data = gtk.ListStore(str, str)
+        self.renderer.connect('editing-started', self.editing_started)
+        self._last_domain = None
+        self._domain_cache = {}
         selection = self.attrs.get('selection', [])[:]
-        self.selection = selection[:]
-        if not self.attrs.get('domain'):
-            domain = []
-        else:
-            domain = PYSONDecoder(rpc.CONTEXT).decode(self.attrs['domain'])
-        if 'relation' in self.attrs:
-            args = ('model', self.attrs['relation'], 'search_read',
-                    domain, 0, None, None, ['rec_name'], rpc.CONTEXT)
+        if not isinstance(selection, (list, tuple)):
+            args = ('model', self.model_name, selection, rpc.CONTEXT)
             try:
-                result = rpc.execute(*args)
-            except Exception, exception:
-                result = common.process_exception(exception, self.window, *args)
-                if not result:
-                    result = []
-            selection = [(x['id'], x['rec_name']) for x in result]
-            self.selection = selection[:]
-        else:
-            if not isinstance(selection, (list, tuple)):
-                try:
-                    selection = rpc.execute('model',
-                            self.model_name, selection, rpc.CONTEXT)
-                except Exception, exception:
-                    common.process_exception(exception, self.window)
-                    selection = []
-                self.selection = selection[:]
-
-            for dom in common.filter_domain(domain):
-                if dom[1] in ('=', '!='):
-                    todel = []
-                    for i in xrange(len(selection)):
-                        if (dom[1] == '=' \
-                                and selection[i][0] != dom[2]) \
-                                or (dom[1] == '!=' \
-                                and selection[i][0] == dom[2]):
-                            todel.append(i)
-                    for i in todel[::-1]:
-                        del selection[i]
-
+                selection = rpc.execute(*args)
+            except TrytonServerError, exception:
+                selection = (common.process_exception(exception, args) or [])
+        self.selection = selection[:]
         if self.attrs.get('sort', True):
-            selection.sort(lambda x, y: cmp(x[1], y[1]))
-        self._selection = selection
-        for i in selection:
-            selection_data.append(i)
-        self.renderer.set_property('model', selection_data)
-        self.renderer.set_property('text-column', 1)
+            selection.sort(key=operator.itemgetter(1))
+        self.renderer.set_property('model', self.get_model(selection))
+        self.renderer.set_property('text-column', 0)
+
+    def get_model(self, selection):
+        model = gtk.ListStore(gobject.TYPE_STRING)
+        self._selection = {}
+        lst = []
+        for (value, name) in selection:
+            name = str(name)
+            lst.append(name)
+            self._selection[name] = value
+            i = model.append()
+            model.set(i, 0, name)
+        return model
 
     def get_textual_value(self, record):
+        self.update_selection(record)
         value = record[self.field_name].get(record)
         if isinstance(value, (list, tuple)):
             value = value[0]
         return dict(self.selection).get(value, '')
 
-    def value_from_text(self, record, text):
-        res = False
-        for val, txt in self._selection:
-            if txt[:len(text)].lower() == text.lower():
-                if len(txt) == len(text):
-                    return val
-                res = val
-        return res
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        field.set_client(record, self._selection.get(text, False))
+        if callback:
+            callback()
+
+    def editing_started(self, cell, editable, path):
+        store = self.treeview.get_model()
+        record = store.get_value(store.get_iter(path), 0)
+        self.update_selection(record)
+        model = self.get_model(self.selection)
+        editable.set_model(model)
+        # GTK 2.24 and above use a ComboBox instead of a ComboBoxEntry
+        if hasattr(editable, 'set_text_column'):
+            editable.set_text_column(0)
+        completion = gtk.EntryCompletion()
+        #Only available in PyGTK 2.6 and above.
+        if hasattr(completion, 'set_inline_selection'):
+            completion.set_inline_selection(True)
+        completion.set_model(model)
+        editable.get_child().set_completion(completion)
+        completion.set_text_column(0)
+        return False
+
+    def update_selection(self, record):
+        if 'relation' not in self.attrs:
+            return
+        field = record[self.field_name]
+        domain = field.domain_get(record)
+        if str(domain) in self._domain_cache:
+            self.selection = self._domain_cache[str(domain)]
+            self._last_domain = domain
+        if domain != self._last_domain:
+            args = ('model', self.attrs['relation'], 'search_read', domain,
+                0, None, None, ['rec_name'], rpc.CONTEXT)
+            try:
+                result = rpc.execute(*args)
+            except TrytonServerError, exception:
+                result = common.process_exception(exception, *args)
+
+            if isinstance(result, list):
+                selection = [(x['id'], x['rec_name']) for x in result]
+                selection.append((False, ''))
+                self._last_domain = domain
+                self._domain_cache[str(domain)] = selection
+            else:
+                selection = []
+                self._last_domain = None
+        else:
+            selection = self.selection
+        self.selection = selection[:]
 
 
 class Reference(Char):
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None):
         super(Reference, self).__init__(field_name, model_name, treeview,
-                window, attrs=attrs)
+            attrs=attrs)
         self._selection = {}
         selection = attrs.get('selection', [])
         if not isinstance(selection, (list, tuple)):
             try:
                 selection = rpc.execute('model',
                         model_name, selection, rpc.CONTEXT)
-            except Exception, exception:
-                common.process_exception(exception, self.window)
+            except TrytonServerError, exception:
+                common.process_exception(exception)
                 selection = []
-        selection.sort(lambda x, y: cmp(x[1], y[1]))
+        selection.sort(key=operator.itemgetter(1))
         for i, j in selection:
             self._selection[i] = str(j)
 
     def get_textual_value(self, record):
         value = record[self.field_name].get_client(record)
         if not value:
-            model, (obj_id, name) = '', (0, '')
+            model, (obj_id, name) = '', (-1, '')
         else:
             model, (obj_id, name) = value
         if model:
-            if not name and obj_id:
-                args = ('model', model, 'read', obj_id, ['rec_name'],
-                        rpc.CONTEXT)
-                try:
-                    res = rpc.execute(*args)
-                except Exception, exception:
-                    res = common.process_exception(exception, self.window,
-                            *args)
-                if not res:
-                    name = '???'
-                else:
-                    name = res['rec_name']
             return self._selection.get(model, model) + ',' + name
         else:
             return name
 
-    def value_from_text(self, record, text):
-        pass
+    def value_from_text(self, record, text, callback=None):
+        if callback:
+            callback()
 
 
 class ProgressBar(object):
@@ -820,7 +873,7 @@ class ProgressBar(object):
         'top_to_bottom': gtk.PROGRESS_TOP_TO_BOTTOM,
     }
 
-    def __init__(self, field_name, model_name, treeview, window, attrs=None):
+    def __init__(self, field_name, model_name, treeview, attrs=None):
         super(ProgressBar, self).__init__()
         self.field_name = field_name
         self.model_name = model_name
@@ -830,37 +883,36 @@ class ProgressBar(object):
             'left_to_right'), gtk.PROGRESS_LEFT_TO_RIGHT)
         self.renderer.set_property('orientation', orientation)
         self.treeview = treeview
-        self.window = window
 
     def setter(self, column, cell, store, iter):
         record = store.get_value(iter, 0)
         value = float(self.get_textual_value(record) or 0.0)
         cell.set_property('value', value)
-        if isinstance(self.attrs.get('digits'), str):
-            digit = record.expr_eval(self.attrs['digits'])[1]
-        else:
-            digit = self.attrs.get('digits', (16, 2))[1]
+        digit = record.expr_eval(self.attrs.get('digits', (16, 2)))[1]
         text = locale.format('%.' + str(digit) + 'f', value, True)
         cell.set_property('text', text + '%')
 
-    def open_remote(self, record, create, changed=False, text=None):
+    def open_remote(self, record, create, changed=False, text=None,
+            callback=None):
         raise NotImplementedError
 
     def get_textual_value(self, record):
         return record[self.field_name].get_client(record) or ''
 
-    def value_from_text(self, record, text):
-        return float(text)
+    def value_from_text(self, record, text, callback=None):
+        field = record[self.field_name]
+        field.set_client(record, float(text))
+        if callback:
+            callback()
 
 
 class Button(object):
 
-    def __init__(self, treeview, window, screen, attrs=None):
+    def __init__(self, treeview, screen, attrs=None):
         super(Button, self).__init__()
         self.attrs = attrs or {}
         self.renderer = CellRendererButton(attrs.get('string', _('Unknown')))
         self.treeview = treeview
-        self.window = window
         self.screen = screen
 
         self.renderer.connect('clicked', self.button_clicked)
@@ -871,8 +923,8 @@ class Button(object):
         store = self.treeview.get_model()
         record = store.get_value(store.get_iter(path), 0)
 
-        state_changes = PYSONDecoder().decode(record.expr_eval(
-            self.attrs.get('states', {}), check_load=False))
+        state_changes = record.expr_eval(
+            self.attrs.get('states', {}), check_load=False)
         if state_changes.get('invisible') \
                 or state_changes.get('readonly'):
             return True
@@ -881,7 +933,7 @@ class Button(object):
         obj_id = self.screen.save_current()
         if obj_id:
             if not self.attrs.get('confirm', False) or \
-                    common.sur(self.attrs['confirm'], self.window):
+                    common.sur(self.attrs['confirm']):
                 button_type = self.attrs.get('type', 'workflow')
                 ctx = rpc.CONTEXT.copy()
                 ctx.update(record.context_get())
@@ -891,32 +943,31 @@ class Button(object):
                             self.attrs['name'], ctx)
                     try:
                         rpc.execute(*args)
-                    except Exception, exception:
-                        common.process_exception(exception, self.window, *args)
+                    except TrytonServerError, exception:
+                        common.process_exception(exception, *args)
                 elif button_type == 'object':
                     args = ('model', self.screen.model_name,
                             self.attrs['name'], [obj_id], ctx)
                     try:
                         rpc.execute(*args)
-                    except Exception, exception:
-                        common.process_exception(exception, self.window, *args)
+                    except TrytonServerError, exception:
+                        common.process_exception(exception, *args)
                 elif button_type == 'action':
                     action_id = None
                     args = ('model', 'ir.action', 'get_action_id',
                             int(self.attrs['name']), ctx)
                     try:
                         action_id = rpc.execute(*args)
-                    except Exception, exception:
-                        action_id = common.process_exception(exception,
-                                self.window, *args)
+                    except TrytonServerError, exception:
+                        action_id = common.process_exception(exception, *args)
                     if action_id:
                         Action.execute(action_id, {
                             'model': self.screen.model_name,
                             'id': obj_id,
                             'ids': [obj_id],
-                            }, self.window, context=ctx)
+                            }, context=ctx)
                 else:
-                    raise Exception('Unallowed button type')
+                    raise TrytonError('Unallowed button type')
                 self.screen.reload(written=True)
             else:
                 self.screen.display()
